use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
ASPEER
Andrew
Handler
JMAC
Jason
Keedi
Kim
Markdown
McIntosh
Mughal
Pod
Speer
Str
ZMUGHAL
Zakariyya
andrew
bin
blockquoted
doit
env
jmac
keedi
lib
markdown2pod
script
zmughal
김도형
